
local GameValueConfig = luajava.bindClass(GNS .. "GameValueConfig")

local setupListener
local disposeListener

local roadTiles = {}
local gvProjSpeed
local gvAttackSpeed
local gvUpgradeLevel

local enemySystemListener = luajava.createProxy(GNS .. "systems.EnemySystem$EnemySystemListener", {
    enemyDie = function(enemy, tower, damageType, fromAbility)
        local explosion = managers.ExplosionManager:getFactory(enums.ExplosionType.CANNON):obtain();

        explosion:setup(tower, enemy.position.x, enemy.position.y, enemy.maxHealth * 0.05, 1.25, 0)
        systems.ExplosionSystem:register(explosion)
        explosion:explode()
    end,

    enemyReachedTarget = function(enemy)

    end,

    affectsGameState = function() return true end,
    getConstantId = function() return 1000901 end
})

local mapSystemListener = luajava.createProxy(GNS .. "systems.MapSystem$MapSystemListener", {
    enemySpawnedOnMap = function(enemy)
        if enemy.killScore == 0 then return nil end

        local road = roadTiles[utils.syncRandInt(#roadTiles) + 1]
        cmd.addenemy(
                enemy.type,
                enemy:getSpeed(),
                enemy.maxHealth,
                0,
                0,
                enemy:getKillExp(),
                road:getX(),
                road:getY(),
                -1)
    end,

    affectsGameState = function() return true end,
    getConstantId = function() return 1000902 end
})

local minerSystemListener = luajava.createProxy(GNS .. "systems.MinerSystem$MinerSystemListener", {
    minerResourcesChanged = function(miner, resourceType, delta, mined)
        if mined then
            systems.GameStateSystem:removeResources(resourceType, delta);

            if resourceType == enums.ResourceType.SCALAR then
                systems.GameStateSystem:addMoney(5, true)
            elseif resourceType == enums.ResourceType.VECTOR then
                gvProjSpeed.value = gvProjSpeed.value + 1
            elseif resourceType == enums.ResourceType.MATRIX then
                gvAttackSpeed.value = gvAttackSpeed.value + 1
            elseif resourceType == enums.ResourceType.TENSOR then
                systems.GameStateSystem:addHealth(delta)
            elseif resourceType == enums.ResourceType.INFIAR then
                if gvUpgradeLevel.value < 7 then
                    gvUpgradeLevel.value = gvUpgradeLevel.value + 1
                end
            end

            systems.GameValueSystem:recalculate()
        end
    end,

    affectsGameState = function() return true end,
    getConstantId = function() return 1000903 end
})

setupListener = function()
    local tiles = systems.MapSystem:getMap().tilesArray
    for i = 1, tiles.size do
        local tile = tiles.items[i]
        if tile.type == enums.TileType.ROAD then
            table.insert(roadTiles, tile)
        end
    end
    log((#roadTiles) .. " road tiles found")

    gvProjSpeed = luajava.newInstance(GNS .. "GameValueConfig", enums.GameValueType.TOWERS_PROJECTILE_SPEED, 0, false, false)
    systems.MapSystem:getMap().targetTile:addGameValue(gvProjSpeed)
    gvAttackSpeed = luajava.newInstance(GNS .. "GameValueConfig", enums.GameValueType.TOWERS_ATTACK_SPEED, 0, false, false)
    systems.MapSystem:getMap().targetTile:addGameValue(gvAttackSpeed)
    gvUpgradeLevel = luajava.newInstance(GNS .. "GameValueConfig", enums.GameValueType.TOWERS_MAX_UPGRADE_LEVEL, 0, false, false)
    systems.MapSystem:getMap().targetTile:addGameValue(gvUpgradeLevel)

    addEventHandler("SystemDispose", disposeListener)

    systems.EnemySystem.listeners:add(enemySystemListener)
    systems.MapSystem.listeners:add(mapSystemListener)
    systems.MinerSystem.listeners:add(minerSystemListener)
end

disposeListener = function()
    removeEventHandler("SystemPostSetup", setupListener)
    removeEventHandler("SystemDispose", disposeListener)

    systems.EnemySystem.listeners:remove(enemySystemListener)
    systems.MapSystem.listeners:remove(mapSystemListener)
    systems.MinerSystem.listeners:remove(minerSystemListener)
end

addEventHandler("SystemPostSetup", setupListener)